@@ -1,29 +1,37 @@
 from __future__ import annotations
 
 import asyncio
+import copy
+import inspect
 import warnings
-from collections.abc import Generator
+import weakref
+from collections.abc import Generator, Mapping
 from contextlib import contextmanager
 from contextvars import ContextVar, Token
 from dataclasses import dataclass
 from enum import Enum
-from typing import Any, Literal, TypeVar, cast, get_origin, overload
+from typing import Any, Literal, cast, get_origin, overload
 
 from mcp import LoggingLevel, ServerSession
 from mcp.server.lowlevel.helper_types import ReadResourceContents
 from mcp.server.lowlevel.server import request_ctx
 from mcp.shared.context import RequestContext
 from mcp.types import (
+    ClientCapabilities,
     ContentBlock,
     CreateMessageResult,
+    IncludeContext,
     ModelHint,
     ModelPreferences,
     Root,
+    SamplingCapability,
     SamplingMessage,
     TextContent,
 )
+from mcp.types import CreateMessageRequestParams as SamplingParams
 from pydantic.networks import AnyUrl
 from starlette.requests import Request
+from typing_extensions import TypeVar
 
 import fastmcp.server.dependencies
 from fastmcp import settings
@@ -40,11 +48,23 @@ from fastmcp.utilities.types import get_cached_typeadapter
 
 logger = get_logger(__name__)
 
-T = TypeVar("T")
-_current_context: ContextVar[Context | None] = ContextVar("context", default=None)
+T = TypeVar("T", default=Any)
+_current_context: ContextVar[Context | None] = ContextVar("context", default=None)  # type: ignore[assignment]
 _flush_lock = asyncio.Lock()
 
 
+@dataclass
+class LogData:
+    """Data object for passing log arguments to client-side handlers.
+
+    This provides an interface to match the Python standard library logging,
+    for compatibility with structured logging.
+    """
+
+    msg: str
+    extra: Mapping[str, Any] | None = None
+
+
 @contextmanager
 def set_context(context: Context) -> Generator[Context, None, None]:
     token = _current_context.set(context)
@@ -82,21 +102,45 @@ class Context:
         request_id = ctx.request_id
         client_id = ctx.client_id
 
+        # Manage state across the request
+        ctx.set_state("key", "value")
+        value = ctx.get_state("key")
+
         return str(x)
     ```
 
+    State Management:
+    Context objects maintain a state dictionary that can be used to store and share
+    data across middleware and tool calls within a request. When a new context
+    is created (nested contexts), it inherits a copy of its parent's state, ensuring
+    that modifications in child contexts don't affect parent contexts.
+
     The context parameter name can be anything as long as it's annotated with Context.
     The context is optional - tools that don't need it can omit the parameter.
 
     """
 
     def __init__(self, fastmcp: FastMCP):
-        self.fastmcp = fastmcp
+        self._fastmcp: weakref.ref[FastMCP] = weakref.ref(fastmcp)
         self._tokens: list[Token] = []
         self._notification_queue: set[str] = set()  # Dedupe notifications
+        self._state: dict[str, Any] = {}
+
+    @property
+    def fastmcp(self) -> FastMCP:
+        """Get the FastMCP instance."""
+        fastmcp = self._fastmcp()
+        if fastmcp is None:
+            raise RuntimeError("FastMCP instance is no longer available")
+        return fastmcp
 
     async def __aenter__(self) -> Context:
         """Enter the context manager and set this context as the current context."""
+        parent_context = _current_context.get(None)
+        if parent_context is not None:
+            # Inherit state from parent context
+            self._state = copy.deepcopy(parent_context._state)
+
         # Always set this context and save the token
         token = _current_context.set(self)
         self._tokens.append(token)
@@ -112,7 +156,7 @@ class Context:
             _current_context.reset(token)
 
     @property
-    def request_context(self) -> RequestContext:
+    def request_context(self) -> RequestContext[ServerSession, Any, Request]:
         """Access to the underlying request context.
 
         If called outside of a request context, this will raise a ValueError.
@@ -158,7 +202,8 @@ class Context:
         Returns:
             The resource content as either text or bytes
         """
-        assert self.fastmcp is not None, "Context is not available outside of a request"
+        if self.fastmcp is None:
+            raise ValueError("Context is not available outside of a request")
         return await self.fastmcp._mcp_read_resource(uri)
 
     async def log(
@@ -166,6 +211,7 @@ class Context:
         message: str,
         level: LoggingLevel | None = None,
         logger_name: str | None = None,
+        extra: Mapping[str, Any] | None = None,
     ) -> None:
         """Send a log message to the client.
 
@@ -174,12 +220,14 @@ class Context:
             level: Optional log level. One of "debug", "info", "notice", "warning", "error", "critical",
                 "alert", or "emergency". Default is "info".
             logger_name: Optional logger name
+            extra: Optional mapping for additional arguments
         """
         if level is None:
             level = "info"
+        data = LogData(msg=message, extra=extra)
         await self.session.send_log_message(
             level=level,
-            data=message,
+            data=data,
             logger=logger_name,
             related_request_id=self.request_id,
         )
@@ -199,35 +247,48 @@ class Context:
         return str(self.request_context.request_id)
 
     @property
-    def session_id(self) -> str | None:
-        """Get the MCP session ID for HTTP transports.
+    def session_id(self) -> str:
+        """Get the MCP session ID for ALL transports.
 
         Returns the session ID that can be used as a key for session-based
         data storage (e.g., Redis) to share data between tool calls within
         the same client session.
 
         Returns:
-            The session ID for HTTP transports (SSE, StreamableHTTP), or None
-            for stdio and in-memory transports which don't use session IDs.
+            The session ID for StreamableHTTP transports, or a generated ID
+            for other transports.
 
         Example:
             ```python
             @server.tool
             def store_data(data: dict, ctx: Context) -> str:
-                if session_id := ctx.session_id:
-                    redis_client.set(f"session:{session_id}:data", json.dumps(data))
-                    return f"Data stored for session {session_id}"
-                return "No session ID available (stdio/memory transport)"
+                session_id = ctx.session_id
+                redis_client.set(f"session:{session_id}:data", json.dumps(data))
+                return f"Data stored for session {session_id}"
             ```
         """
-        try:
-            from fastmcp.server.dependencies import get_http_headers
+        request_ctx = self.request_context
+        session = request_ctx.session
 
-            headers = get_http_headers(include_all=True)
-            return headers.get("mcp-session-id")
-        except RuntimeError:
-            # No HTTP context available (stdio/in-memory transport)
-            return None
+        # Try to get the session ID from the session attributes
+        session_id = getattr(session, "_fastmcp_id", None)
+        if session_id is not None:
+            return session_id
+
+        # Try to get the session ID from the http request headers
+        request = request_ctx.request
+        if request:
+            session_id = request.headers.get("mcp-session-id")
+
+        # Generate a session ID if it doesn't exist.
+        if session_id is None:
+            from uuid import uuid4
+
+            session_id = str(uuid4())
+
+        # Save the session id to the session attributes
+        setattr(session, "_fastmcp_id", session_id)
+        return session_id
 
     @property
     def session(self) -> ServerSession:
@@ -235,21 +296,49 @@ class Context:
         return self.request_context.session
 
     # Convenience methods for common log levels
-    async def debug(self, message: str, logger_name: str | None = None) -> None:
+    async def debug(
+        self,
+        message: str,
+        logger_name: str | None = None,
+        extra: Mapping[str, Any] | None = None,
+    ) -> None:
         """Send a debug log message."""
-        await self.log(level="debug", message=message, logger_name=logger_name)
+        await self.log(
+            level="debug", message=message, logger_name=logger_name, extra=extra
+        )
 
-    async def info(self, message: str, logger_name: str | None = None) -> None:
+    async def info(
+        self,
+        message: str,
+        logger_name: str | None = None,
+        extra: Mapping[str, Any] | None = None,
+    ) -> None:
         """Send an info log message."""
-        await self.log(level="info", message=message, logger_name=logger_name)
+        await self.log(
+            level="info", message=message, logger_name=logger_name, extra=extra
+        )
 
-    async def warning(self, message: str, logger_name: str | None = None) -> None:
+    async def warning(
+        self,
+        message: str,
+        logger_name: str | None = None,
+        extra: Mapping[str, Any] | None = None,
+    ) -> None:
         """Send a warning log message."""
-        await self.log(level="warning", message=message, logger_name=logger_name)
+        await self.log(
+            level="warning", message=message, logger_name=logger_name, extra=extra
+        )
 
-    async def error(self, message: str, logger_name: str | None = None) -> None:
+    async def error(
+        self,
+        message: str,
+        logger_name: str | None = None,
+        extra: Mapping[str, Any] | None = None,
+    ) -> None:
         """Send an error log message."""
-        await self.log(level="error", message=message, logger_name=logger_name)
+        await self.log(
+            level="error", message=message, logger_name=logger_name, extra=extra
+        )
 
     async def list_roots(self) -> list[Root]:
         """List the roots available to the server, as indicated by the client."""
@@ -272,6 +361,7 @@ class Context:
         self,
         messages: str | list[str | SamplingMessage],
         system_prompt: str | None = None,
+        include_context: IncludeContext | None = None,
         temperature: float | None = None,
         max_tokens: int | None = None,
         model_preferences: ModelPreferences | str | list[str] | None = None,
@@ -301,12 +391,50 @@ class Context:
                 for m in messages
             ]
 
+        should_fallback = (
+            self.fastmcp.sampling_handler_behavior == "fallback"
+            and not self.session.check_client_capability(
+                capability=ClientCapabilities(sampling=SamplingCapability())
+            )
+        )
+
+        if self.fastmcp.sampling_handler_behavior == "always" or should_fallback:
+            if self.fastmcp.sampling_handler is None:
+                raise ValueError("Client does not support sampling")
+
+            create_message_result = self.fastmcp.sampling_handler(
+                sampling_messages,
+                SamplingParams(
+                    systemPrompt=system_prompt,
+                    messages=sampling_messages,
+                    temperature=temperature,
+                    maxTokens=max_tokens,
+                    modelPreferences=_parse_model_preferences(model_preferences),
+                ),
+                self.request_context,
+            )
+
+            if inspect.isawaitable(create_message_result):
+                create_message_result = await create_message_result
+
+            if isinstance(create_message_result, str):
+                return TextContent(text=create_message_result, type="text")
+
+            if isinstance(create_message_result, CreateMessageResult):
+                return create_message_result.content
+
+            else:
+                raise ValueError(
+                    f"Unexpected sampling handler result: {create_message_result}"
+                )
+
         result: CreateMessageResult = await self.session.create_message(
             messages=sampling_messages,
             system_prompt=system_prompt,
+            include_context=include_context,
             temperature=temperature,
             max_tokens=max_tokens,
-            model_preferences=self._parse_model_preferences(model_preferences),
+            model_preferences=_parse_model_preferences(model_preferences),
             related_request_id=self.request_id,
         )
 
@@ -421,7 +549,7 @@ class Context:
                 if isinstance(validated_data, ScalarElicitationType):
                     return AcceptedElicitation[T](data=validated_data.value)
                 else:
-                    return AcceptedElicitation[T](data=validated_data)
+                    return AcceptedElicitation[T](data=cast(T, validated_data))
             elif result.content:
                 raise ValueError(
                     "Elicitation expected an empty response, but received: "
@@ -452,6 +580,14 @@ class Context:
 
         return fastmcp.server.dependencies.get_http_request()
 
+    def set_state(self, key: str, value: Any) -> None:
+        """Set a value in the context state."""
+        self._state[key] = value
+
+    def get_state(self, key: str) -> Any:
+        """Get a value from the context state. Returns None if the key is not found."""
+        return self._state.get(key)
+
     def _queue_tool_list_changed(self) -> None:
         """Queue a tool list changed notification."""
         self._notification_queue.add("notifications/tools/list_changed")
@@ -498,44 +634,43 @@ class Context:
                 # Don't let notification failures break the request
                 pass
 
-    def _parse_model_preferences(
-        self, model_preferences: ModelPreferences | str | list[str] | None
-    ) -> ModelPreferences | None:
-        """
-        Validates and converts user input for model_preferences into a ModelPreferences object.
 
-        Args:
-            model_preferences (ModelPreferences | str | list[str] | None):
-                The model preferences to use. Accepts:
-                - ModelPreferences (returns as-is)
-                - str (single model hint)
-                - list[str] (multiple model hints)
-                - None (no preferences)
+def _parse_model_preferences(
+    model_preferences: ModelPreferences | str | list[str] | None,
+) -> ModelPreferences | None:
+    """
+    Validates and converts user input for model_preferences into a ModelPreferences object.
 
-        Returns:
-            ModelPreferences | None: The parsed ModelPreferences object, or None if not provided.
+    Args:
+        model_preferences (ModelPreferences | str | list[str] | None):
+            The model preferences to use. Accepts:
+            - ModelPreferences (returns as-is)
+            - str (single model hint)
+            - list[str] (multiple model hints)
+            - None (no preferences)
 
-        Raises:
-            ValueError: If the input is not a supported type or contains invalid values.
-        """
-        if model_preferences is None:
-            return None
-        elif isinstance(model_preferences, ModelPreferences):
-            return model_preferences
-        elif isinstance(model_preferences, str):
-            # Single model hint
-            return ModelPreferences(hints=[ModelHint(name=model_preferences)])
-        elif isinstance(model_preferences, list):
-            # List of model hints (strings)
-            if not all(isinstance(h, str) for h in model_preferences):
-                raise ValueError(
-                    "All elements of model_preferences list must be"
-                    " strings (model name hints)."
-                )
-            return ModelPreferences(
-                hints=[ModelHint(name=h) for h in model_preferences]
-            )
-        else:
+    Returns:
+        ModelPreferences | None: The parsed ModelPreferences object, or None if not provided.
+
+    Raises:
+        ValueError: If the input is not a supported type or contains invalid values.
+    """
+    if model_preferences is None:
+        return None
+    elif isinstance(model_preferences, ModelPreferences):
+        return model_preferences
+    elif isinstance(model_preferences, str):
+        # Single model hint
+        return ModelPreferences(hints=[ModelHint(name=model_preferences)])
+    elif isinstance(model_preferences, list):
+        # List of model hints (strings)
+        if not all(isinstance(h, str) for h in model_preferences):
             raise ValueError(
-                "model_preferences must be one of: ModelPreferences, str, list[str], or None."
+                "All elements of model_preferences list must be"
+                " strings (model name hints)."
             )
+        return ModelPreferences(hints=[ModelHint(name=h) for h in model_preferences])
+    else:
+        raise ValueError(
+            "model_preferences must be one of: ModelPreferences, str, list[str], or None."
+        )
