@@ -15,6 +15,7 @@ from fastmcp.server.elicitation import (
     AcceptedElicitation,
     CancelledElicitation,
     DeclinedElicitation,
+    get_elicitation_schema,
     validate_elicitation_json_schema,
 )
 from fastmcp.utilities.types import TypeAdapter
@@ -547,6 +548,8 @@ class TestPatternMatching:
                     return "You declined"
                 case CancelledElicitation():
                     return "Cancelled"
+                case _:
+                    return "Unknown result"
 
         async def elicitation_handler(message, response_type, params, ctx):
             return ElicitResult(action="accept", content={"value": "Alice"})
@@ -570,6 +573,8 @@ class TestPatternMatching:
                     return "You declined"
                 case CancelledElicitation():
                     return "Cancelled"
+                case _:
+                    return "Unknown result"
 
         async def elicitation_handler(message, response_type, params, ctx):
             return ElicitResult(action="decline")
@@ -593,6 +598,8 @@ class TestPatternMatching:
                     return "You declined"
                 case CancelledElicitation():
                     return "Cancelled"
+                case _:
+                    return "Unknown result"
 
         async def elicitation_handler(message, response_type, params, ctx):
             return ElicitResult(action="cancel")
@@ -633,3 +640,80 @@ async def test_elicitation_implicit_acceptance_must_be_dict(fastmcp_server):
             match="Elicitation responses must be serializable as a JSON object",
         ):
             await client.call_tool("ask_for_name")
+
+
+def test_enum_elicitation_schema_inline():
+    """Test that enum schemas are generated inline without $ref/$defs for MCP compatibility."""
+
+    class Priority(Enum):
+        LOW = "low"
+        MEDIUM = "medium"
+        HIGH = "high"
+
+    @dataclass
+    class TaskRequest:
+        title: str
+        priority: Priority
+
+    # Generate elicitation schema
+    schema = get_elicitation_schema(TaskRequest)
+
+    # Verify no $defs section exists (enums should be inlined)
+    assert "$defs" not in schema, (
+        "Schema should not contain $defs - enums must be inline"
+    )
+
+    # Verify no $ref in properties
+    for prop_name, prop_schema in schema.get("properties", {}).items():
+        assert "$ref" not in prop_schema, (
+            f"Property {prop_name} contains $ref - should be inline"
+        )
+
+    # Verify the priority field has inline enum values
+    priority_schema = schema["properties"]["priority"]
+    assert "enum" in priority_schema, "Priority should have enum values inline"
+    assert priority_schema["enum"] == ["low", "medium", "high"]
+    assert priority_schema.get("type") == "string"
+
+    # Verify title field is a simple string
+    assert schema["properties"]["title"]["type"] == "string"
+
+
+def test_enum_elicitation_schema_with_enum_names():
+    """Test that enum schemas can include enumNames for better UI display."""
+
+    class TaskStatus(Enum):
+        NOT_STARTED = "not_started"
+        IN_PROGRESS = "in_progress"
+        COMPLETED = "completed"
+        ON_HOLD = "on_hold"
+
+    @dataclass
+    class TaskUpdate:
+        task_id: str
+        status: TaskStatus
+
+    # Generate elicitation schema
+    schema = get_elicitation_schema(TaskUpdate)
+
+    # Verify enum is inline
+    assert "$defs" not in schema
+    assert "$ref" not in str(schema)
+
+    status_schema = schema["properties"]["status"]
+    assert "enum" in status_schema
+    assert status_schema["enum"] == [
+        "not_started",
+        "in_progress",
+        "completed",
+        "on_hold",
+    ]
+
+    # Check if enumNames were added for display
+    assert "enumNames" in status_schema
+    assert status_schema["enumNames"] == [
+        "Not Started",
+        "In Progress",
+        "Completed",
+        "On Hold",
+    ]
