@@ -3,14 +3,15 @@ from typing import Annotated
 import pytest
 
 from fastmcp import Context
-from fastmcp.exceptions import NotFoundError
+from fastmcp.exceptions import NotFoundError, PromptError
 from fastmcp.prompts import Prompt
-from fastmcp.prompts.prompt import PromptMessage, TextContent
+from fastmcp.prompts.prompt import FunctionPrompt, PromptMessage, TextContent
 from fastmcp.prompts.prompt_manager import PromptManager
+from fastmcp.utilities.tests import caplog_for_fastmcp
 
 
 class TestPromptManager:
-    def test_add_prompt(self):
+    async def test_add_prompt(self):
         """Test adding a prompt to the manager."""
 
         def fn() -> str:
@@ -20,9 +21,9 @@ class TestPromptManager:
         prompt = Prompt.from_function(fn)
         added = manager.add_prompt(prompt)
         assert added == prompt
-        assert manager.get_prompt("fn") == prompt
+        assert await manager.get_prompt("fn") == prompt
 
-    def test_add_duplicate_prompt(self, caplog):
+    async def test_add_duplicate_prompt(self, caplog):
         """Test adding the same prompt twice."""
 
         def fn() -> str:
@@ -31,11 +32,14 @@ class TestPromptManager:
         manager = PromptManager(duplicate_behavior="warn")
         prompt = Prompt.from_function(fn)
         first = manager.add_prompt(prompt)
-        second = manager.add_prompt(prompt)
+
+        with caplog_for_fastmcp(caplog):
+            second = manager.add_prompt(prompt)
+
         assert first == second
         assert "Prompt already exists" in caplog.text
 
-    def test_disable_warn_on_duplicate_prompts(self, caplog):
+    async def test_disable_warn_on_duplicate_prompts(self, caplog):
         """Test disabling warning on duplicate prompts."""
 
         def fn() -> str:
@@ -48,7 +52,7 @@ class TestPromptManager:
         assert first == second
         assert "Prompt already exists" not in caplog.text
 
-    def test_warn_on_duplicate_prompts(self, caplog):
+    async def test_warn_on_duplicate_prompts(self, caplog):
         """Test warning on duplicate prompts."""
         manager = PromptManager(duplicate_behavior="warn")
 
@@ -58,13 +62,15 @@ class TestPromptManager:
         prompt = Prompt.from_function(test_fn, name="test_prompt")
 
         manager.add_prompt(prompt)
-        manager.add_prompt(prompt)
+
+        with caplog_for_fastmcp(caplog):
+            manager.add_prompt(prompt)
 
         assert "Prompt already exists: test_prompt" in caplog.text
         # Should have the prompt
-        assert manager.get_prompt("test_prompt") is not None
+        assert await manager.get_prompt("test_prompt") is not None
 
-    def test_error_on_duplicate_prompts(self):
+    async def test_error_on_duplicate_prompts(self):
         """Test error on duplicate prompts."""
         manager = PromptManager(duplicate_behavior="error")
 
@@ -78,7 +84,7 @@ class TestPromptManager:
         with pytest.raises(ValueError, match="Prompt already exists: test_prompt"):
             manager.add_prompt(prompt)
 
-    def test_replace_duplicate_prompts(self):
+    async def test_replace_duplicate_prompts(self):
         """Test replacing duplicate prompts."""
         manager = PromptManager(duplicate_behavior="replace")
 
@@ -95,11 +101,12 @@ class TestPromptManager:
         manager.add_prompt(prompt2)
 
         # Should have replaced with the new prompt
-        prompt = manager.get_prompt("test_prompt")
+        prompt = await manager.get_prompt("test_prompt")
         assert prompt is not None
+        assert isinstance(prompt, FunctionPrompt)
         assert prompt.fn.__name__ == "replacement_fn"
 
-    def test_ignore_duplicate_prompts(self):
+    async def test_ignore_duplicate_prompts(self):
         """Test ignoring duplicate prompts."""
         manager = PromptManager(duplicate_behavior="ignore")
 
@@ -116,13 +123,15 @@ class TestPromptManager:
         result = manager.add_prompt(prompt2)
 
         # Should keep the original
-        prompt = manager.get_prompt("test_prompt")
+        prompt = await manager.get_prompt("test_prompt")
         assert prompt is not None
+        assert isinstance(prompt, FunctionPrompt)
         assert prompt.fn.__name__ == "original_fn"
         # Result should be the original prompt
+        assert isinstance(result, FunctionPrompt)
         assert result.fn.__name__ == "original_fn"
 
-    def test_get_prompts(self):
+    async def test_get_prompts(self):
         """Test retrieving all prompts."""
 
         def fn1() -> str:
@@ -136,11 +145,13 @@ class TestPromptManager:
         prompt2 = Prompt.from_function(fn2)
         manager.add_prompt(prompt1)
         manager.add_prompt(prompt2)
-        prompts = manager.get_prompts()
+        prompts = await manager.get_prompts()
         assert len(prompts) == 2
         assert prompts["fn1"] == prompt1
         assert prompts["fn2"] == prompt2
 
+
+class TestRenderPrompt:
     async def test_render_prompt(self):
         """Test rendering a prompt."""
 
@@ -177,6 +188,48 @@ class TestPromptManager:
             )
         ]
 
+    async def test_render_prompt_callable_object(self):
+        """Test rendering a prompt with a callable object."""
+
+        class MyPrompt:
+            """A callable object that can be used as a prompt."""
+
+            def __call__(self, name: str) -> str:
+                """ignore this"""
+                return f"Hello, {name}!"
+
+        manager = PromptManager()
+        prompt = Prompt.from_function(MyPrompt())
+        manager.add_prompt(prompt)
+        result = await manager.render_prompt("MyPrompt", arguments={"name": "World"})
+        assert result.description == "A callable object that can be used as a prompt."
+        assert result.messages == [
+            PromptMessage(
+                role="user", content=TextContent(type="text", text="Hello, World!")
+            )
+        ]
+
+    async def test_render_prompt_callable_object_async(self):
+        """Test rendering a prompt with a callable object."""
+
+        class MyPrompt:
+            """A callable object that can be used as a prompt."""
+
+            async def __call__(self, name: str) -> str:
+                """ignore this"""
+                return f"Hello, {name}!"
+
+        manager = PromptManager()
+        prompt = Prompt.from_function(MyPrompt())
+        manager.add_prompt(prompt)
+        result = await manager.render_prompt("MyPrompt", arguments={"name": "World"})
+        assert result.description == "A callable object that can be used as a prompt."
+        assert result.messages == [
+            PromptMessage(
+                role="user", content=TextContent(type="text", text="Hello, World!")
+            )
+        ]
+
     async def test_render_unknown_prompt(self):
         """Test rendering a non-existent prompt."""
         manager = PromptManager()
@@ -192,7 +245,7 @@ class TestPromptManager:
         manager = PromptManager()
         prompt = Prompt.from_function(fn)
         manager.add_prompt(prompt)
-        with pytest.raises(ValueError, match="Missing required arguments"):
+        with pytest.raises(PromptError, match="Missing required arguments"):
             await manager.render_prompt("fn")
 
     async def test_prompt_with_varargs_not_allowed(self):
@@ -223,7 +276,7 @@ class TestPromptManager:
 class TestPromptTags:
     """Test functionality related to prompt tags."""
 
-    def test_add_prompt_with_tags(self):
+    async def test_add_prompt_with_tags(self):
         """Test adding a prompt with tags."""
 
         def greeting() -> str:
@@ -233,11 +286,11 @@ class TestPromptTags:
         prompt = Prompt.from_function(greeting, tags={"greeting", "simple"})
         manager.add_prompt(prompt)
 
-        prompt = manager.get_prompt("greeting")
+        prompt = await manager.get_prompt("greeting")
         assert prompt is not None
         assert prompt.tags == {"greeting", "simple"}
 
-    def test_add_prompt_with_empty_tags(self):
+    async def test_add_prompt_with_empty_tags(self):
         """Test adding a prompt with empty tags."""
 
         def greeting() -> str:
@@ -247,11 +300,11 @@ class TestPromptTags:
         prompt = Prompt.from_function(greeting, tags=set())
         manager.add_prompt(prompt)
 
-        prompt = manager.get_prompt("greeting")
+        prompt = await manager.get_prompt("greeting")
         assert prompt is not None
         assert prompt.tags == set()
 
-    def test_add_prompt_with_none_tags(self):
+    async def test_add_prompt_with_none_tags(self):
         """Test adding a prompt with None tags."""
 
         def greeting() -> str:
@@ -261,11 +314,11 @@ class TestPromptTags:
         prompt = Prompt.from_function(greeting, tags=None)
         manager.add_prompt(prompt)
 
-        prompt = manager.get_prompt("greeting")
+        prompt = await manager.get_prompt("greeting")
         assert prompt is not None
         assert prompt.tags == set()
 
-    def test_list_prompts_with_tags(self):
+    async def test_list_prompts_with_tags(self):
         """Test listing prompts with specific tags."""
 
         def greeting() -> str:
@@ -285,13 +338,12 @@ class TestPromptTags:
         )
 
         # Filter prompts by tags
-        simple_prompts = [
-            p for p in manager.get_prompts().values() if "simple" in p.tags
-        ]
+        prompts = await manager.get_prompts()
+        simple_prompts = [p for p in prompts.values() if "simple" in p.tags]
         assert len(simple_prompts) == 2
         assert {p.name for p in simple_prompts} == {"greeting", "summary"}
 
-        nlp_prompts = [p for p in manager.get_prompts().values() if "nlp" in p.tags]
+        nlp_prompts = [p for p in prompts.values() if "nlp" in p.tags]
         assert len(nlp_prompts) == 1
         assert nlp_prompts[0].name == "summary"
 
@@ -345,12 +397,11 @@ class TestContextHandling:
         mcp = FastMCP()
         context = Context(fastmcp=mcp)
 
-        with context:
+        async with context:
             messages = await prompt.render(arguments={"x": 42})
 
         assert len(messages) == 1
-        assert isinstance(messages[0].content, TextContent)
-        assert messages[0].content.text == "42"
+        assert messages[0].content.text == "42"  # type: ignore[attr-defined]
 
     async def test_context_optional(self):
         """Test that context is optional when rendering prompts."""
@@ -366,14 +417,13 @@ class TestContextHandling:
         mcp = FastMCP()
         context = Context(fastmcp=mcp)
 
-        with context:
+        async with context:
             messages = await prompt.render(
                 arguments={"x": 42},
             )
 
         assert len(messages) == 1
-        assert isinstance(messages[0].content, TextContent)
-        assert messages[0].content.text == "42"
+        assert messages[0].content.text == "42"  # type: ignore[attr-defined]
 
     async def test_annotated_context_parameter_detection(self):
         """Test that annotated context parameters are properly detected in
