@@ -1,5 +1,6 @@
+import inspect
 import json
-from typing import Any
+from typing import Any, cast
 
 import pytest
 from anyio import create_task_group
@@ -12,6 +13,9 @@ from fastmcp.client import Client
 from fastmcp.client.transports import FastMCPTransport, StreamableHttpTransport
 from fastmcp.exceptions import ToolError
 from fastmcp.server.proxy import FastMCPProxy, ProxyClient
+from fastmcp.tools.tool_transform import (
+    ToolTransformConfig,
+)
 
 USERS = [
     {"id": "1", "name": "Alice", "active": True},
@@ -26,7 +30,7 @@ def fastmcp_server():
 
     # --- Tools ---
 
-    @server.tool
+    @server.tool(tags={"greet"})
     def greet(name: str) -> str:
         """Greet someone by name."""
         return f"Hello, {name}!"
@@ -47,7 +51,7 @@ def fastmcp_server():
 
     # --- Resources ---
 
-    @server.resource(uri="resource://wave")
+    @server.resource(uri="resource://wave", tags={"wave"})
     def wave() -> str:
         return "👋"
 
@@ -55,13 +59,13 @@ def fastmcp_server():
     async def get_users() -> list[dict[str, Any]]:
         return USERS
 
-    @server.resource(uri="data://user/{user_id}")
+    @server.resource(uri="data://user/{user_id}", tags={"users"})
     async def get_user(user_id: str) -> dict[str, Any] | None:
         return next((user for user in USERS if user["id"] == user_id), None)
 
     # --- Prompts ---
 
-    @server.prompt
+    @server.prompt(tags={"welcome"})
     def welcome(name: str) -> str:
         return f"Welcome to FastMCP, {name}!"
 
@@ -83,7 +87,7 @@ async def test_create_proxy(fastmcp_server):
 
     assert isinstance(server, FastMCPProxy)
     assert isinstance(server, FastMCP)
-    assert server.name == "FastMCP"
+    assert server.name.startswith("FastMCPProxy-")
 
 
 async def test_as_proxy_with_server(fastmcp_server):
@@ -106,8 +110,24 @@ def test_as_proxy_with_url():
     """FastMCP.as_proxy should accept a URL without connecting."""
     proxy = FastMCP.as_proxy("http://example.com/mcp/")
     assert isinstance(proxy, FastMCPProxy)
-    assert isinstance(proxy.client_factory().transport, StreamableHttpTransport)
-    assert proxy.client_factory().transport.url == "http://example.com/mcp/"  # type: ignore[attr-defined]
+    client = cast(Client, proxy.client_factory())
+    assert isinstance(client.transport, StreamableHttpTransport)
+    assert client.transport.url == "http://example.com/mcp/"  # type: ignore[attr-defined]
+
+
+async def test_proxy_with_async_client_factory():
+    """FastMCPProxy should accept an async client_factory."""
+
+    async def async_factory():
+        return Client("http://example.com/mcp/")
+
+    proxy = FastMCPProxy(client_factory=async_factory)
+    assert isinstance(proxy, FastMCPProxy)
+    assert inspect.iscoroutinefunction(proxy.client_factory)
+    client = await proxy.client_factory()
+    assert isinstance(client, Client)
+    assert isinstance(client.transport, StreamableHttpTransport)
+    assert client.transport.url == "http://example.com/mcp/"  # type: ignore[attr-defined]
 
 
 class TestTools:
@@ -118,6 +138,35 @@ class TestTools:
         assert "error_tool" in tools
         assert "tool_without_description" in tools
 
+    async def test_get_tools_meta(self, proxy_server):
+        tools = await proxy_server.get_tools()
+        greet_tool = tools["greet"]
+        assert greet_tool.meta == {"_fastmcp": {"tags": ["greet"]}}
+
+    async def test_get_transformed_tools(
+        self, fastmcp_server: FastMCP, proxy_server: FastMCPProxy
+    ):
+        """An explicit None description should change the tool description to None."""
+
+        fastmcp_server.add_tool_transformation(
+            "add", ToolTransformConfig(name="add_transformed")
+        )
+        tools = await proxy_server.get_tools()
+        assert "add_transformed" in tools
+        assert "add" not in tools
+
+    async def test_call_transformed_tools(
+        self, fastmcp_server: FastMCP, proxy_server: FastMCPProxy
+    ):
+        """An explicit None description should change the tool description to None."""
+
+        fastmcp_server.add_tool_transformation(
+            "add", ToolTransformConfig(name="add_transformed")
+        )
+        async with Client(proxy_server) as client:
+            result = await client.call_tool("add_transformed", {"a": 1, "b": 2})
+        assert result.data == 3
+
     async def test_tool_without_description(self, proxy_server):
         tools = await proxy_server.get_tools()
         assert tools["tool_without_description"].description is None
@@ -211,6 +260,11 @@ class TestResources:
         )
         assert [r.name for r in resources.values()] == Contains("get_users", "wave")
 
+    async def test_get_resources_meta(self, proxy_server):
+        resources = await proxy_server.get_resources()
+        wave_resource = resources["resource://wave"]
+        assert wave_resource.meta == {"_fastmcp": {"tags": ["wave"]}}
+
     async def test_list_resources_same_as_original(self, fastmcp_server, proxy_server):
         assert (
             await proxy_server._mcp_list_resources()
@@ -305,6 +359,11 @@ class TestResourceTemplates:
         templates = await proxy_server.get_resource_templates()
         assert [t.name for t in templates.values()] == Contains("get_user")
 
+    async def test_get_resource_templates_meta(self, proxy_server):
+        templates = await proxy_server.get_resource_templates()
+        get_user_template = templates["data://user/{user_id}"]
+        assert get_user_template.meta == {"_fastmcp": {"tags": ["users"]}}
+
     async def test_list_resource_templates_same_as_original(
         self, fastmcp_server, proxy_server
     ):
@@ -404,6 +463,11 @@ class TestPrompts:
         prompts = await proxy_server.get_prompts()
         assert [p.name for p in prompts.values()] == Contains("welcome")
 
+    async def test_get_prompts_meta(self, proxy_server):
+        prompts = await proxy_server.get_prompts()
+        welcome_prompt = prompts["welcome"]
+        assert welcome_prompt.meta == {"_fastmcp": {"tags": ["welcome"]}}
+
     async def test_list_prompts_same_as_original(self, fastmcp_server, proxy_server):
         async with Client(fastmcp_server) as client:
             result = await client.list_prompts()
@@ -516,3 +580,161 @@ async def test_proxy_handles_multiple_concurrent_tasks_correctly(
     assert list(results["tools"]) == Contains(
         "greet", "add", "error_tool", "tool_without_description"
     )
+
+
+class TestMirroredComponents:
+    """Test mirrored component functionality - components retrieved from proxy servers."""
+
+    async def test_mirrored_tool_cannot_be_enabled(self, proxy_server):
+        """Test that mirrored tools cannot be enabled directly."""
+        tools = await proxy_server.get_tools()
+        mirrored_tool = tools["greet"]
+
+        # Verify it's mirrored
+        assert mirrored_tool._mirrored is True
+
+        # Should raise error when trying to enable
+        with pytest.raises(RuntimeError, match="Cannot enable mirrored component"):
+            mirrored_tool.enable()
+
+    async def test_mirrored_tool_cannot_be_disabled(self, proxy_server):
+        """Test that mirrored tools cannot be disabled directly."""
+        tools = await proxy_server.get_tools()
+        mirrored_tool = tools["greet"]
+
+        # Verify it's mirrored
+        assert mirrored_tool._mirrored is True
+
+        # Should raise error when trying to disable
+        with pytest.raises(RuntimeError, match="Cannot disable mirrored component"):
+            mirrored_tool.disable()
+
+    async def test_mirrored_resource_cannot_be_enabled(self, proxy_server):
+        """Test that mirrored resources cannot be enabled directly."""
+        resources = await proxy_server.get_resources()
+        mirrored_resource = resources["resource://wave"]
+
+        # Verify it's mirrored
+        assert mirrored_resource._mirrored is True
+
+        # Should raise error when trying to enable
+        with pytest.raises(RuntimeError, match="Cannot enable mirrored component"):
+            mirrored_resource.enable()
+
+    async def test_mirrored_resource_cannot_be_disabled(self, proxy_server):
+        """Test that mirrored resources cannot be disabled directly."""
+        resources = await proxy_server.get_resources()
+        mirrored_resource = resources["resource://wave"]
+
+        # Verify it's mirrored
+        assert mirrored_resource._mirrored is True
+
+        # Should raise error when trying to disable
+        with pytest.raises(RuntimeError, match="Cannot disable mirrored component"):
+            mirrored_resource.disable()
+
+    async def test_mirrored_prompt_cannot_be_enabled(self, proxy_server):
+        """Test that mirrored prompts cannot be enabled directly."""
+        prompts = await proxy_server.get_prompts()
+        mirrored_prompt = prompts["welcome"]
+
+        # Verify it's mirrored
+        assert mirrored_prompt._mirrored is True
+
+        # Should raise error when trying to enable
+        with pytest.raises(RuntimeError, match="Cannot enable mirrored component"):
+            mirrored_prompt.enable()
+
+    async def test_mirrored_prompt_cannot_be_disabled(self, proxy_server):
+        """Test that mirrored prompts cannot be disabled directly."""
+        prompts = await proxy_server.get_prompts()
+        mirrored_prompt = prompts["welcome"]
+
+        # Verify it's mirrored
+        assert mirrored_prompt._mirrored is True
+
+        # Should raise error when trying to disable
+        with pytest.raises(RuntimeError, match="Cannot disable mirrored component"):
+            mirrored_prompt.disable()
+
+    async def test_copy_creates_non_mirrored_component(self, proxy_server):
+        """Test that copy() creates a non-mirrored component that can be modified."""
+        tools = await proxy_server.get_tools()
+        mirrored_tool = tools["greet"]
+
+        # Create a copy
+        local_tool = mirrored_tool.copy()
+
+        # Copy should not be mirrored
+        assert local_tool._mirrored is False
+
+        # Should be able to enable/disable the copy
+        local_tool.enable()
+        assert local_tool.enabled is True
+
+        local_tool.disable()
+        assert local_tool.enabled is False
+
+    async def test_local_component_takes_precedence_over_mirrored(self, proxy_server):
+        """Test that local components take precedence over mirrored ones."""
+        # Get the mirrored tool
+        tools = await proxy_server.get_tools()
+        mirrored_tool = tools["greet"]
+
+        # Create a local copy and add it
+        local_tool = mirrored_tool.copy()
+        proxy_server.add_tool(local_tool)
+
+        # Disable the local copy
+        local_tool.disable()
+
+        # The local disabled tool should take precedence
+        updated_tools = await proxy_server.get_tools()
+        final_tool = updated_tools["greet"]
+
+        # Should be the local tool (not mirrored) and disabled
+        assert final_tool is local_tool
+        assert final_tool._mirrored is False
+        assert final_tool.enabled is False
+
+    async def test_error_messages_mention_copy_method(self, proxy_server):
+        """Test that error messages guide users to use copy() method."""
+        tools = await proxy_server.get_tools()
+        mirrored_tool = tools["greet"]
+
+        # Check enable error message
+        with pytest.raises(RuntimeError) as exc_info:
+            mirrored_tool.enable()
+        assert "copy()" in str(exc_info.value)
+
+        # Check disable error message
+        with pytest.raises(RuntimeError) as exc_info:
+            mirrored_tool.disable()
+        assert "copy()" in str(exc_info.value)
+
+    async def test_client_cannot_call_disabled_proxy_tool(self, proxy_server):
+        """Test that clients cannot call a tool when local copy is disabled."""
+        # Get the mirrored tool
+        tools = await proxy_server.get_tools()
+        mirrored_tool = tools["greet"]
+
+        # Verify the tool works initially
+        async with Client(proxy_server) as client:
+            result = await client.call_tool("greet", {"name": "Alice"})
+            assert result.data == "Hello, Alice!"
+
+        # Create a local copy and disable it
+        local_tool = mirrored_tool.copy()
+        proxy_server.add_tool(local_tool)
+        local_tool.disable()
+
+        # Client should now get "Unknown tool" error
+        async with Client(proxy_server) as client:
+            with pytest.raises(ToolError, match="Unknown tool"):
+                await client.call_tool("greet", {"name": "Alice"})
+
+        # Tool should not appear in tool list either
+        async with Client(proxy_server) as client:
+            tools_list = await client.list_tools()
+            tool_names = [tool.name for tool in tools_list]
+            assert "greet" not in tool_names
