@@ -3,6 +3,8 @@ from dataclasses import dataclass
 from typing import Annotated, Any
 
 import pytest
+from dirty_equals import HasName
+from inline_snapshot import snapshot
 from mcp.types import (
     AudioContent,
     EmbeddedResource,
@@ -13,7 +15,7 @@ from mcp.types import (
 from pydantic import AnyUrl, BaseModel, Field, TypeAdapter
 from typing_extensions import TypedDict
 
-from fastmcp.tools.tool import Tool, _convert_to_content
+from fastmcp.tools.tool import Tool, ToolResult, _convert_to_content
 from fastmcp.utilities.json_schema import compress_schema
 from fastmcp.utilities.tests import caplog_for_fastmcp
 from fastmcp.utilities.types import Audio, File, Image
@@ -29,20 +31,47 @@ class TestToolFromFunction:
 
         tool = Tool.from_function(add)
 
-        assert tool.name == "add"
-        assert tool.description == "Add two numbers."
-        assert len(tool.parameters["properties"]) == 2
-        assert tool.parameters["properties"]["a"]["type"] == "integer"
-        assert tool.parameters["properties"]["b"]["type"] == "integer"
-        # With primitive wrapping, int return type becomes object with result property
-        expected_schema = {
-            "type": "object",
-            "properties": {"result": {"type": "integer", "title": "Result"}},
-            "required": ["result"],
-            "title": "_WrappedResult",
-            "x-fastmcp-wrap-result": True,
-        }
-        assert tool.output_schema == expected_schema
+        assert tool.model_dump(exclude_none=True) == snapshot(
+            {
+                "name": "add",
+                "description": "Add two numbers.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {
+                        "a": {"title": "A", "type": "integer"},
+                        "b": {"title": "B", "type": "integer"},
+                    },
+                    "required": ["a", "b"],
+                    "type": "object",
+                },
+                "output_schema": {
+                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "required": ["result"],
+                    "title": "_WrappedResult",
+                    "type": "object",
+                    "x-fastmcp-wrap-result": True,
+                },
+                "fn": HasName("add"),
+            }
+        )
+
+    def test_meta_parameter(self):
+        """Test that meta parameter is properly handled."""
+
+        def multiply(a: int, b: int) -> int:
+            """Multiply two numbers."""
+            return a * b
+
+        meta_data = {"version": "1.0", "author": "test"}
+        tool = Tool.from_function(multiply, meta=meta_data)
+
+        assert tool.meta == meta_data
+        mcp_tool = tool.to_mcp_tool()
+
+        # MCP tool includes fastmcp meta, so check that our meta is included
+        assert mcp_tool.meta is not None
+        assert meta_data.items() <= mcp_tool.meta.items()
 
     async def test_async_function(self):
         """Test registering and running an async function."""
@@ -53,9 +82,27 @@ class TestToolFromFunction:
 
         tool = Tool.from_function(fetch_data)
 
-        assert tool.name == "fetch_data"
-        assert tool.description == "Fetch data from URL."
-        assert tool.parameters["properties"]["url"]["type"] == "string"
+        assert tool.model_dump(exclude_none=True) == snapshot(
+            {
+                "name": "fetch_data",
+                "description": "Fetch data from URL.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {"url": {"title": "Url", "type": "string"}},
+                    "required": ["url"],
+                    "type": "object",
+                },
+                "output_schema": {
+                    "properties": {"result": {"title": "Result", "type": "string"}},
+                    "required": ["result"],
+                    "title": "_WrappedResult",
+                    "type": "object",
+                    "x-fastmcp-wrap-result": True,
+                },
+                "fn": HasName("fetch_data"),
+            }
+        )
 
     def test_callable_object(self):
         class Adder:
@@ -66,11 +113,30 @@ class TestToolFromFunction:
                 return x + y
 
         tool = Tool.from_function(Adder())
-        assert tool.name == "Adder"
-        assert tool.description == "Adds two numbers."
-        assert len(tool.parameters["properties"]) == 2
-        assert tool.parameters["properties"]["x"]["type"] == "integer"
-        assert tool.parameters["properties"]["y"]["type"] == "integer"
+
+        assert tool.model_dump(exclude_none=True, exclude={"fn"}) == snapshot(
+            {
+                "name": "Adder",
+                "description": "Adds two numbers.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {
+                        "x": {"title": "X", "type": "integer"},
+                        "y": {"title": "Y", "type": "integer"},
+                    },
+                    "required": ["x", "y"],
+                    "type": "object",
+                },
+                "output_schema": {
+                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "required": ["result"],
+                    "title": "_WrappedResult",
+                    "type": "object",
+                    "x-fastmcp-wrap-result": True,
+                },
+            }
+        )
 
     def test_async_callable_object(self):
         class Adder:
@@ -81,11 +147,30 @@ class TestToolFromFunction:
                 return x + y
 
         tool = Tool.from_function(Adder())
-        assert tool.name == "Adder"
-        assert tool.description == "Adds two numbers."
-        assert len(tool.parameters["properties"]) == 2
-        assert tool.parameters["properties"]["x"]["type"] == "integer"
-        assert tool.parameters["properties"]["y"]["type"] == "integer"
+
+        assert tool.model_dump(exclude_none=True, exclude={"fn"}) == snapshot(
+            {
+                "name": "Adder",
+                "description": "Adds two numbers.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {
+                        "x": {"title": "X", "type": "integer"},
+                        "y": {"title": "Y", "type": "integer"},
+                    },
+                    "required": ["x", "y"],
+                    "type": "object",
+                },
+                "output_schema": {
+                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "required": ["result"],
+                    "title": "_WrappedResult",
+                    "type": "object",
+                    "x-fastmcp-wrap-result": True,
+                },
+            }
+        )
 
     def test_pydantic_model_function(self):
         """Test registering a function that takes a Pydantic model."""
@@ -100,20 +185,45 @@ class TestToolFromFunction:
 
         tool = Tool.from_function(create_user)
 
-        assert tool.name == "create_user"
-        assert tool.description == "Create a new user."
-        assert "name" in tool.parameters["$defs"]["UserInput"]["properties"]
-        assert "age" in tool.parameters["$defs"]["UserInput"]["properties"]
-        assert "flag" in tool.parameters["properties"]
+        assert tool.model_dump(exclude_none=True) == snapshot(
+            {
+                "name": "create_user",
+                "description": "Create a new user.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "$defs": {
+                        "UserInput": {
+                            "properties": {
+                                "name": {"title": "Name", "type": "string"},
+                                "age": {"title": "Age", "type": "integer"},
+                            },
+                            "required": ["name", "age"],
+                            "title": "UserInput",
+                            "type": "object",
+                        }
+                    },
+                    "properties": {
+                        "user": {"$ref": "#/$defs/UserInput", "title": "User"},
+                        "flag": {"title": "Flag", "type": "boolean"},
+                    },
+                    "required": ["user", "flag"],
+                    "type": "object",
+                },
+                "output_schema": {"additionalProperties": True, "type": "object"},
+                "fn": HasName("create_user"),
+            }
+        )
 
     async def test_tool_with_image_return(self):
         def image_tool(data: bytes) -> Image:
             return Image(data=data)
 
         tool = Tool.from_function(image_tool)
+        assert tool.parameters["properties"]["data"]["type"] == "string"
+        assert tool.output_schema is None
 
         result = await tool.run({"data": "test.png"})
-        assert tool.parameters["properties"]["data"]["type"] == "string"
         assert isinstance(result.content[0], ImageContent)
 
     async def test_tool_with_audio_return(self):
@@ -121,9 +231,10 @@ class TestToolFromFunction:
             return Audio(data=data)
 
         tool = Tool.from_function(audio_tool)
+        assert tool.parameters["properties"]["data"]["type"] == "string"
+        assert tool.output_schema is None
 
         result = await tool.run({"data": "test.wav"})
-        assert tool.parameters["properties"]["data"]["type"] == "string"
         assert isinstance(result.content[0], AudioContent)
 
     async def test_tool_with_file_return(self):
@@ -131,15 +242,20 @@ class TestToolFromFunction:
             return File(data=data, format="octet-stream")
 
         tool = Tool.from_function(file_tool)
-
-        result = await tool.run({"data": "test.bin"})
         assert tool.parameters["properties"]["data"]["type"] == "string"
-        assert len(result.content) == 1
-        assert isinstance(result.content[0], EmbeddedResource)
-        assert result.content[0].type == "resource"
-        assert hasattr(result.content[0], "resource")
-        resource = result.content[0].resource
-        assert resource.mimeType == "application/octet-stream"
+        assert tool.output_schema is None
+
+        result: ToolResult = await tool.run({"data": "test.bin"})
+        assert result.content[0].model_dump(exclude_none=True) == snapshot(
+            {
+                "type": "resource",
+                "resource": {
+                    "uri": AnyUrl("file:///resource.octet-stream"),
+                    "mimeType": "application/octet-stream",
+                    "blob": "dGVzdC5iaW4=",
+                },
+            }
+        )
 
     def test_non_callable_fn(self):
         with pytest.raises(TypeError, match="not a callable object"):
@@ -147,7 +263,18 @@ class TestToolFromFunction:
 
     def test_lambda(self):
         tool = Tool.from_function(lambda x: x, name="my_tool")
-        assert tool.name == "my_tool"
+        assert tool.model_dump(exclude_none=True, exclude={"fn"}) == snapshot(
+            {
+                "name": "my_tool",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {"x": {"title": "X"}},
+                    "required": ["x"],
+                    "type": "object",
+                },
+            }
+        )
 
     def test_lambda_with_no_name(self):
         with pytest.raises(
@@ -161,8 +288,25 @@ class TestToolFromFunction:
             return _a + _b
 
         tool = Tool.from_function(add)
-        assert tool.parameters["properties"]["_a"]["type"] == "integer"
-        assert tool.parameters["properties"]["_b"]["type"] == "integer"
+
+        assert tool.model_dump(
+            exclude_none=True, exclude={"output_schema", "fn"}
+        ) == snapshot(
+            {
+                "name": "add",
+                "description": "Add two numbers.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {
+                        "_a": {"title": "A", "type": "integer"},
+                        "_b": {"title": "B", "type": "integer"},
+                    },
+                    "required": ["_a", "_b"],
+                    "type": "object",
+                },
+            }
+        )
 
     def test_tool_with_varargs_not_allowed(self):
         def func(a: int, b: int, *args: int) -> int:
@@ -193,10 +337,32 @@ class TestToolFromFunction:
         obj = MyClass()
 
         tool = Tool.from_function(obj.add)
-        assert tool.name == "add"
-        assert tool.description == "Add two numbers."
         assert "self" not in tool.parameters["properties"]
 
+        assert tool.model_dump(exclude_none=True, exclude={"fn"}) == snapshot(
+            {
+                "name": "add",
+                "description": "Add two numbers.",
+                "tags": set(),
+                "enabled": True,
+                "parameters": {
+                    "properties": {
+                        "x": {"title": "X", "type": "integer"},
+                        "y": {"title": "Y", "type": "integer"},
+                    },
+                    "required": ["x", "y"],
+                    "type": "object",
+                },
+                "output_schema": {
+                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "required": ["result"],
+                    "title": "_WrappedResult",
+                    "type": "object",
+                    "x-fastmcp-wrap-result": True,
+                },
+            }
+        )
+
     async def test_instance_method_with_varargs_not_allowed(self):
         class MyClass:
             def add(self, x: int, y: int, *args: int) -> int:
@@ -306,6 +472,7 @@ class TestToolFromFunctionOutputSchema:
                 "x-fastmcp-wrap-result": True,
             }
             assert tool.output_schema == expected_schema
+            # # Note: Parameterized test - keeping original assertion for multiple parameter values
         else:
             # Object types remain unwrapped
             assert tool.output_schema == base_schema
@@ -323,8 +490,8 @@ class TestToolFromFunctionOutputSchema:
             return 1
 
         tool = Tool.from_function(func)
-        base_schema = TypeAdapter(annotation).json_schema()
 
+        base_schema = TypeAdapter(annotation).json_schema()
         expected_schema = {
             "type": "object",
             "properties": {"result": {**base_schema, "title": "Result"}},
@@ -389,8 +556,18 @@ class TestToolFromFunctionOutputSchema:
             return Person(name="John", age=30)
 
         tool = Tool.from_function(func)
-        expected_schema = compress_schema(TypeAdapter(Person).json_schema())
-        assert tool.output_schema == expected_schema
+
+        assert tool.output_schema == snapshot(
+            {
+                "properties": {
+                    "name": {"title": "Name", "type": "string"},
+                    "age": {"title": "Age", "type": "integer"},
+                },
+                "required": ["name", "age"],
+                "title": "Person",
+                "type": "object",
+            }
+        )
 
     async def test_typeddict_return_annotation(self):
         class Person(TypedDict):
@@ -401,8 +578,17 @@ class TestToolFromFunctionOutputSchema:
             return Person(name="John", age=30)
 
         tool = Tool.from_function(func)
-        expected_schema = compress_schema(TypeAdapter(Person).json_schema())
-        assert tool.output_schema == expected_schema
+        assert tool.output_schema == snapshot(
+            {
+                "properties": {
+                    "name": {"title": "Name", "type": "string"},
+                    "age": {"title": "Age", "type": "integer"},
+                },
+                "required": ["name", "age"],
+                "title": "Person",
+                "type": "object",
+            }
+        )
 
     async def test_unserializable_return_annotation(self):
         class Unserializable:
@@ -546,7 +732,7 @@ class TestToolFromFunctionOutputSchema:
         def func() -> dict[str, str]:
             return {"message": "Hello, world!"}
 
-        tool = Tool.from_function(func, output_schema=False)
+        tool = Tool.from_function(func, output_schema=None)
         assert tool.output_schema is None
 
         result = await tool.run({})
@@ -577,14 +763,15 @@ class TestToolFromFunctionOutputSchema:
 
         # Don't specify output_schema - should infer and wrap
         tool = Tool.from_function(func)
-        expected_schema = {
-            "type": "object",
-            "properties": {"result": {"type": "integer", "title": "Result"}},
-            "required": ["result"],
-            "title": "_WrappedResult",
-            "x-fastmcp-wrap-result": True,
-        }
-        assert tool.output_schema == expected_schema
+        assert tool.output_schema == snapshot(
+            {
+                "properties": {"result": {"title": "Result", "type": "integer"}},
+                "required": ["result"],
+                "title": "_WrappedResult",
+                "type": "object",
+                "x-fastmcp-wrap-result": True,
+            }
+        )
 
         result = await tool.run({})
         assert result.structured_content == {"result": 42}
@@ -649,14 +836,15 @@ class TestToolFromFunctionOutputSchema:
 
         # Inferred schema should wrap string type
         tool = Tool.from_function(func)
-        expected_schema = {
-            "type": "object",
-            "properties": {"result": {"type": "string", "title": "Result"}},
-            "required": ["result"],
-            "title": "_WrappedResult",
-            "x-fastmcp-wrap-result": True,
-        }
-        assert tool.output_schema == expected_schema
+        assert tool.output_schema == snapshot(
+            {
+                "properties": {"result": {"title": "Result", "type": "string"}},
+                "required": ["result"],
+                "title": "_WrappedResult",
+                "type": "object",
+                "x-fastmcp-wrap-result": True,
+            }
+        )
 
         result = await tool.run({})
         # Unstructured content
@@ -713,17 +901,23 @@ class TestToolFromFunctionOutputSchema:
         tool_none = Tool.from_function(func, output_schema=None)
         assert tool_none.output_schema is None
 
-        # False should also disable
-        tool_false = Tool.from_function(func, output_schema=False)
-        assert tool_false.output_schema is None
+        # Default (NotSet) should infer from return type
+        tool_default = Tool.from_function(func)
+        assert (
+            tool_default.output_schema is not None
+        )  # Should infer schema from dict return type
 
-        # Both should have same behavior
+        # Different behavior: None vs inferred
         result_none = await tool_none.run({})
-        result_false = await tool_false.run({})
+        result_default = await tool_default.run({})
 
-        assert result_none.structured_content is None
-        assert result_false.structured_content is None
-        assert result_none.content[0].text == result_false.content[0].text == "123"  # type: ignore[attr-defined]
+        # None should still try fallback generation but fail for non-dict
+        assert result_none.structured_content is None  # Fallback fails for int
+        # Default should use proper schema and wrap the result
+        assert result_default.structured_content == {
+            "result": 123
+        }  # Schema-based generation with wrapping
+        assert result_none.content[0].text == result_default.content[0].text == "123"  # type: ignore[attr-defined]
 
     async def test_non_object_output_schema_raises_error(self):
         """Test that providing a non-object output schema raises a ValueError."""
@@ -1101,7 +1295,7 @@ class TestAutomaticStructuredContent:
             return UserProfile(name="Bob", age=25, email="bob@example.com")
 
         # No explicit output schema, but dataclass should still create structured content
-        tool = Tool.from_function(get_profile, output_schema=False)
+        tool = Tool.from_function(get_profile, output_schema=None)
 
         result = await tool.run({"user_id": "456"})
 
@@ -1128,8 +1322,8 @@ class TestAutomaticStructuredContent:
         def get_user_stats(user_id: str) -> UserData:
             return UserData(username="charlie", score=100, verified=True)
 
-        # Explicitly disable output schema to test automatic structured content
-        tool = Tool.from_function(get_user_stats, output_schema=False)
+        # Explicitly set output schema to None to test automatic structured content
+        tool = Tool.from_function(get_user_stats, output_schema=None)
 
         result = await tool.run({"user_id": "789"})
 
@@ -1291,6 +1485,88 @@ class TestUnionReturnTypes:
         assert result2.structured_content == {"result": "error occurred"}
 
 
+class TestSerializationAlias:
+    """Tests for Pydantic field serialization alias support in tool output schemas."""
+
+    def test_output_schema_respects_serialization_alias(self):
+        """Test that Tool.from_function generates output schema using serialization alias."""
+        from pydantic import AliasChoices, BaseModel, Field
+
+        class Component(BaseModel):
+            """Model with multiple validation aliases but specific serialization alias."""
+
+            component_id: str = Field(
+                validation_alias=AliasChoices("id", "componentId"),
+                serialization_alias="componentId",
+                description="The ID of the component",
+            )
+
+        async def get_component(
+            component_id: str,
+        ) -> Annotated[Component, Field(description="The component.")]:
+            # API returns data with 'id' field
+            api_data = {"id": component_id}
+            return Component.model_validate(api_data)
+
+        tool = Tool.from_function(get_component, name="get-component")
+
+        # The output schema should use the serialization alias 'componentId'
+        # not the first validation alias 'id'
+        assert tool.output_schema is not None
+
+        # Check the wrapped result schema
+        assert "properties" in tool.output_schema
+        assert "result" in tool.output_schema["properties"]
+        assert "$defs" in tool.output_schema
+
+        # Find the Component definition
+        component_def = list(tool.output_schema["$defs"].values())[0]
+
+        # Should have 'componentId' not 'id' in properties
+        assert "componentId" in component_def["properties"]
+        assert "id" not in component_def["properties"]
+
+        # Should require 'componentId' not 'id'
+        assert "componentId" in component_def["required"]
+        assert "id" not in component_def.get("required", [])
+
+    async def test_tool_execution_with_serialization_alias(self):
+        """Test that tool execution works correctly with serialization aliases."""
+        from pydantic import AliasChoices, BaseModel, Field
+
+        from fastmcp import Client, FastMCP
+
+        class Component(BaseModel):
+            """Model with multiple validation aliases but specific serialization alias."""
+
+            component_id: str = Field(
+                validation_alias=AliasChoices("id", "componentId"),
+                serialization_alias="componentId",
+                description="The ID of the component",
+            )
+
+        mcp = FastMCP("TestServer")
+
+        @mcp.tool
+        async def get_component(
+            component_id: str,
+        ) -> Annotated[Component, Field(description="The component.")]:
+            # API returns data with 'id' field
+            api_data = {"id": component_id}
+            return Component.model_validate(api_data)
+
+        async with Client(mcp) as client:
+            # Execute the tool - this should work without validation errors
+            result = await client.call_tool(
+                "get_component", {"component_id": "test123"}
+            )
+
+            # The result should contain the serialized form with 'componentId'
+            assert result.structured_content is not None
+            assert result.structured_content["result"]["componentId"] == "test123"
+            assert "id" not in result.structured_content["result"]
+
+
 class TestToolTitle:
     """Tests for tool title functionality."""
 
