@@ -12,7 +12,8 @@ from fastmcp import Context, FastMCP
 from fastmcp.exceptions import NotFoundError, ToolError
 from fastmcp.tools import FunctionTool, ToolManager
 from fastmcp.tools.tool import Tool
-from fastmcp.utilities.tests import caplog_for_fastmcp
+from fastmcp.tools.tool_transform import ArgTransformConfig, ToolTransformConfig
+from fastmcp.utilities.tests import caplog_for_fastmcp, temporary_settings
 from fastmcp.utilities.types import Image
 
 
@@ -262,6 +263,52 @@ class TestAddTools:
         assert result.fn.__name__ == "replacement_fn"
 
 
+class TestListTools:
+    async def test_list_tools_with_transformed_names(self):
+        """Test listing tools with transformations."""
+
+        tool_manager = ToolManager()
+
+        def add(a: int, b: int) -> int:
+            return a + b
+
+        tool = Tool.from_function(add)
+        tool_manager.add_tool(tool)
+
+        tool_manager.add_tool_transformation(
+            "add", ToolTransformConfig(name="add_transformed")
+        )
+        tools = await tool_manager.list_tools()
+        tools_by_name = {tool.name: tool for tool in tools}
+        assert "add_transformed" in tools_by_name
+        assert "add" not in tools_by_name
+
+    async def test_list_tools_with_transforms(self):
+        """Test listing tools with transformations."""
+
+        tool_manager = ToolManager()
+
+        def add(a: int, b: int) -> int:
+            """Add two numbers."""
+            return a + b
+
+        tool = Tool.from_function(add)
+        tool_manager.add_tool(tool)
+
+        tool_manager.add_tool_transformation(
+            "add",
+            ToolTransformConfig(
+                name="add_transformed", description=None, tags={"enabled_tools"}
+            ),
+        )
+        tools = await tool_manager.list_tools()
+        tools_by_name = {tool.name: tool for tool in tools}
+        assert "add_transformed" in tools_by_name
+        assert "add" not in tools_by_name
+        assert tools_by_name["add_transformed"].description is None
+        assert tools_by_name["add_transformed"].tags == {"enabled_tools"}
+
+
 class TestToolTags:
     """Test functionality related to tool tags."""
 
@@ -431,6 +478,39 @@ class TestCallTools:
         with pytest.raises(NotFoundError, match="Tool 'unknown' not found"):
             await manager.call_tool("unknown", {"a": 1})
 
+    async def test_call_transformed_tool(self):
+        manager = ToolManager()
+
+        def add(a: int, b: int) -> int:
+            """Add two numbers."""
+            return a + b
+
+        tool = Tool.from_function(add)
+        manager.add_tool(tool)
+
+        manager.add_tool_transformation(
+            "add",
+            ToolTransformConfig(
+                name="add_transformed",
+                description=None,
+                tags={"enabled_tools"},
+                arguments={
+                    "a": ArgTransformConfig(
+                        name="a_transformed", description=None, default=1
+                    ),
+                    "b": ArgTransformConfig(
+                        name="b_transformed", description=None, default=2
+                    ),
+                },
+            ),
+        )
+
+        result = await manager.call_tool(
+            "add_transformed", {"a_transformed": 1, "b_transformed": 2}
+        )
+        assert result.content[0].text == "3"  # type: ignore[attr-defined]
+        assert result.structured_content == {"result": 3}
+
     async def test_call_tool_with_list_int_input(self):
         def sum_vals(vals: list[int]) -> int:
             return sum(vals)
@@ -754,8 +834,8 @@ class TestCustomToolNames:
         # Create a tool with a specific name
         tool = Tool.from_function(fn, name="my_tool")
         manager = ToolManager()
-        # Use with_key to create a new tool with the custom key
-        tool_with_custom_key = tool.with_key("proxy_tool")
+        # Use model_copy to create a new tool with the custom key
+        tool_with_custom_key = tool.model_copy(key="proxy_tool")
         manager.add_tool(tool_with_custom_key)
         # The tool is accessible under the key
         stored = await manager.get_tool("proxy_tool")
@@ -916,3 +996,44 @@ class TestToolErrorHandling:
         # Exception message should contain the tool name but not the internal details
         assert "Error calling tool 'async_buggy_tool'" in str(excinfo.value)
         assert "Internal async error details" not in str(excinfo.value)
+
+
+class TestMountedComponentsRaiseOnLoadError:
+    """Test the mounted_components_raise_on_load_error setting."""
+
+    async def test_mounted_components_raise_on_load_error_default_false(self):
+        """Test that by default, mounted component load errors are warned and not raised."""
+        import fastmcp
+
+        # Ensure default setting is False
+        assert fastmcp.settings.mounted_components_raise_on_load_error is False
+
+        parent_mcp = FastMCP("ParentServer")
+        child_mcp = FastMCP("FailingChildServer")
+
+        # Create a failing mounted server by corrupting it
+        parent_mcp.mount(child_mcp, prefix="child")
+        # Corrupt the child server to make it fail during tool loading
+        child_mcp._tool_manager._mounted_servers.append("invalid")  # type: ignore
+
+        # Should not raise, just warn
+        tools = await parent_mcp._tool_manager.list_tools()
+        assert isinstance(tools, list)  # Should return empty list, not raise
+
+    async def test_mounted_components_raise_on_load_error_true(self):
+        """Test that when enabled, mounted component load errors are raised."""
+        parent_mcp = FastMCP("ParentServer")
+        child_mcp = FastMCP("FailingChildServer")
+
+        # Create a failing mounted server
+        parent_mcp.mount(child_mcp, prefix="child")
+        # Corrupt the child server to make it fail during tool loading
+        child_mcp._tool_manager._mounted_servers.append("invalid")  # type: ignore
+
+        # Use temporary settings context manager
+        with temporary_settings(mounted_components_raise_on_load_error=True):
+            # Should raise the exception
+            with pytest.raises(
+                AttributeError, match="'str' object has no attribute 'server'"
+            ):
+                await parent_mcp._tool_manager.list_tools()
