@@ -4,6 +4,7 @@ from typing import Annotated, Any
 
 import pytest
 from dirty_equals import IsList
+from inline_snapshot import snapshot
 from mcp.types import TextContent
 from pydantic import BaseModel, Field, TypeAdapter
 from typing_extensions import TypedDict
@@ -13,7 +14,11 @@ from fastmcp.client.client import Client
 from fastmcp.exceptions import ToolError
 from fastmcp.tools import Tool, forward, forward_raw
 from fastmcp.tools.tool import FunctionTool, ToolResult
-from fastmcp.tools.tool_transform import ArgTransform, TransformedTool
+from fastmcp.tools.tool_transform import (
+    ArgTransform,
+    ToolTransformConfig,
+    TransformedTool,
+)
 
 
 def get_property(tool: Tool, name: str) -> dict[str, Any]:
@@ -192,6 +197,31 @@ async def test_hide_required_param_with_user_default_works():
     assert result.structured_content == {"result": 25}
 
 
+async def test_hidden_param_prunes_defs():
+    class VisibleType(BaseModel):
+        x: int
+
+    class HiddenType(BaseModel):
+        y: int
+
+    @Tool.from_function
+    def tool_with_refs(a: VisibleType, b: HiddenType | None = None) -> int:
+        return a.x + (b.y if b else 0)
+
+    # Hide parameter 'b'
+    new_tool = Tool.from_tool(
+        tool_with_refs, transform_args={"b": ArgTransform(hide=True)}
+    )
+
+    schema = new_tool.parameters
+    # Only 'a' should be visible
+    assert list(schema["properties"].keys()) == ["a"]
+    # $defs should only contain VisibleType, not HiddenType
+    defs = schema.get("$defs", {})
+    assert "VisibleType" in defs
+    assert "HiddenType" not in defs
+
+
 async def test_forward_with_argument_mapping(add_tool):
     """Test that forward() applies argument mapping correctly."""
 
@@ -404,11 +434,13 @@ def test_transform_args_validation_unknown_arg(add_tool):
     """Test that transform_args with unknown arguments raises ValueError."""
     with pytest.raises(
         ValueError, match="Unknown arguments in transform_args: unknown_param"
-    ):
+    ) as exc_info:
         Tool.from_tool(
             add_tool, transform_args={"unknown_param": ArgTransform(name="new_name")}
         )
 
+    assert "`add`" in str(exc_info.value)
+
 
 def test_transform_args_creates_duplicate_names(add_tool):
     """Test that transform_args creating duplicate parameter names raises ValueError."""
@@ -989,7 +1021,12 @@ class TestEnableDisable:
         new_add = Tool.from_tool(add, name="new_add")
         mcp.add_tool(new_add)
 
-        assert new_add.enabled
+        # the new tool inherits the disabled state from the parent tool
+        assert new_add.enabled is False
+
+        new_add.enable()
+        assert new_add.enabled is True
+        assert add.enabled is False
 
         async with Client(mcp) as client:
             tools = await client.list_tools()
@@ -1019,38 +1056,6 @@ class TestEnableDisable:
                 await client.call_tool("new_add", {"x": 1, "y": 2})
 
 
-def test_arg_transform_examples_in_schema(add_tool):
-    # Simple example
-    new_tool = Tool.from_tool(
-        add_tool,
-        transform_args={
-            "old_x": ArgTransform(examples=[1, 2, 3]),
-        },
-    )
-    prop = get_property(new_tool, "old_x")
-    assert prop["examples"] == [1, 2, 3]
-
-    # Nested example (e.g., for array type)
-    new_tool2 = Tool.from_tool(
-        add_tool,
-        transform_args={
-            "old_x": ArgTransform(examples=[["a", "b"], ["c", "d"]]),
-        },
-    )
-    prop2 = get_property(new_tool2, "old_x")
-    assert prop2["examples"] == [["a", "b"], ["c", "d"]]
-
-    # If not set, should not be present
-    new_tool3 = Tool.from_tool(
-        add_tool,
-        transform_args={
-            "old_x": ArgTransform(),
-        },
-    )
-    prop3 = get_property(new_tool3, "old_x")
-    assert "examples" not in prop3
-
-
 class TestTransformToolOutputSchema:
     """Test output schema handling in transformed tools."""
 
@@ -1087,15 +1092,15 @@ class TestTransformToolOutputSchema:
         assert new_tool.output_schema == expected_schema
         assert new_tool.output_schema == base_string_tool.output_schema
 
-    def test_transform_with_explicit_output_schema_false(self, base_string_tool):
-        """Test that output_schema=False disables structured output."""
-        new_tool = Tool.from_tool(base_string_tool, output_schema=False)
+    def test_transform_with_explicit_output_schema_none(self, base_string_tool):
+        """Test that output_schema=None sets output schema to None."""
+        new_tool = Tool.from_tool(base_string_tool, output_schema=None)
 
         assert new_tool.output_schema is None
 
-    async def test_transform_output_schema_false_runtime(self, base_string_tool):
-        """Test runtime behavior with output_schema=False."""
-        new_tool = Tool.from_tool(base_string_tool, output_schema=False)
+    async def test_transform_output_schema_none_runtime(self, base_string_tool):
+        """Test runtime behavior with output_schema=None."""
+        new_tool = Tool.from_tool(base_string_tool, output_schema=None)
 
         # Debug: check that output_schema is actually None
         assert new_tool.output_schema is None, (
@@ -1103,7 +1108,8 @@ class TestTransformToolOutputSchema:
         )
 
         result = await new_tool.run({"x": 5})
-        assert result.structured_content is None
+        # Even with output_schema=None, structured content should be generated via fallback logic
+        assert result.structured_content == {"result": "Result: 5"}
         assert result.content[0].text == "Result: 5"  # type: ignore[attr-defined]
 
     def test_transform_with_explicit_output_schema_dict(self, base_string_tool):
@@ -1275,25 +1281,27 @@ class TestTransformToolOutputSchema:
         expected_schema = TypeAdapter(dict[str, str]).json_schema()
         assert new_tool.output_schema == expected_schema
 
-    async def test_transform_output_schema_none_vs_false(self, base_string_tool):
-        """Test None vs False behavior for output_schema in transforms."""
-        # None (default) should use smart fallback (inherit from parent)
-        tool_none = Tool.from_tool(base_string_tool)  # default output_schema=None
-        assert tool_none.output_schema == base_string_tool.output_schema  # Inherits
+    async def test_transform_output_schema_default_vs_none(self, base_string_tool):
+        """Test default (NotSet) vs explicit None behavior for output_schema in transforms."""
+        # Default (NotSet) should use smart fallback (inherit from parent)
+        tool_default = Tool.from_tool(base_string_tool)  # default output_schema=NotSet
+        assert tool_default.output_schema == base_string_tool.output_schema  # Inherits
 
-        # False should explicitly disable
-        tool_false = Tool.from_tool(base_string_tool, output_schema=False)
-        assert tool_false.output_schema is None
+        # None should explicitly set output_schema to None but still generate structured content via fallback
+        tool_explicit_none = Tool.from_tool(base_string_tool, output_schema=None)
+        assert tool_explicit_none.output_schema is None
 
-        # Different behavior at runtime
-        result_none = await tool_none.run({"x": 5})
-        result_false = await tool_false.run({"x": 5})
+        # Both should generate structured content now (via different paths)
+        result_default = await tool_default.run({"x": 5})
+        result_explicit_none = await tool_explicit_none.run({"x": 5})
 
-        assert result_none.structured_content == {
+        assert result_default.structured_content == {
             "result": "Result: 5"
         }  # Inherits wrapping
-        assert result_false.structured_content is None  # Disabled
-        assert result_none.content[0].text == result_false.content[0].text  # type: ignore[attr-defined]
+        assert result_explicit_none.structured_content == {
+            "result": "Result: 5"
+        }  # Generated via fallback logic
+        assert result_default.content[0].text == result_explicit_none.content[0].text  # type: ignore[attr-defined]
 
     async def test_transform_output_schema_with_tool_result_return(
         self, base_string_tool
@@ -1316,3 +1324,386 @@ class TestTransformToolOutputSchema:
         # Should use ToolResult content directly
         assert result.content[0].text == "Direct: 6"  # type: ignore[attr-defined]
         assert result.structured_content == {"direct_value": 6, "doubled": 12}
+
+
+@pytest.fixture
+def sample_tool():
+    """Sample tool for testing transformations."""
+
+    def sample_func(x: int) -> str:
+        return f"Result: {x}"
+
+    return Tool.from_function(
+        sample_func,
+        name="sample_tool",
+        title="Original Tool Title",
+        description="Original description",
+    )
+
+
+@pytest.fixture
+def sample_tool_no_title():
+    """Sample tool without title for testing."""
+
+    def sample_func(x: int) -> str:
+        return f"Result: {x}"
+
+    return Tool.from_function(sample_func, name="no_title_tool")
+
+
+def test_transform_inherits_title(sample_tool):
+    """Test that transformed tools inherit title when none specified."""
+    transformed = Tool.from_tool(sample_tool)
+    assert transformed.title == "Original Tool Title"
+
+
+def test_transform_overrides_title(sample_tool):
+    """Test that transformed tools can override title."""
+    transformed = Tool.from_tool(sample_tool, title="New Tool Title")
+    assert transformed.title == "New Tool Title"
+
+
+def test_transform_sets_title_to_none(sample_tool):
+    """Test that transformed tools can explicitly set title to None."""
+    transformed = Tool.from_tool(sample_tool, title=None)
+    assert transformed.title is None
+
+
+def test_transform_inherits_none_title(sample_tool_no_title):
+    """Test that transformed tools inherit None title."""
+    transformed = Tool.from_tool(sample_tool_no_title)
+    assert transformed.title is None
+
+
+def test_transform_adds_title_to_none(sample_tool_no_title):
+    """Test that transformed tools can add title when parent has None."""
+    transformed = Tool.from_tool(sample_tool_no_title, title="Added Title")
+    assert transformed.title == "Added Title"
+
+
+def test_transform_inherits_description(sample_tool):
+    """Test that transformed tools inherit description when none specified."""
+    transformed = Tool.from_tool(sample_tool)
+    assert transformed.description == "Original description"
+
+
+def test_transform_overrides_description(sample_tool):
+    """Test that transformed tools can override description."""
+    transformed = Tool.from_tool(sample_tool, description="New description")
+    assert transformed.description == "New description"
+
+
+def test_transform_sets_description_to_none(sample_tool):
+    """Test that transformed tools can explicitly set description to None."""
+    transformed = Tool.from_tool(sample_tool, description=None)
+    assert transformed.description is None
+
+
+def test_transform_inherits_none_description(sample_tool_no_title):
+    """Test that transformed tools inherit None description."""
+    transformed = Tool.from_tool(sample_tool_no_title)
+    assert transformed.description is None
+
+
+def test_transform_adds_description_to_none(sample_tool_no_title):
+    """Test that transformed tools can add description when parent has None."""
+    transformed = Tool.from_tool(sample_tool_no_title, description="Added description")
+    assert transformed.description == "Added description"
+
+
+# Meta transformation tests
+def test_transform_inherits_meta(sample_tool):
+    """Test that transformed tools inherit meta when none specified."""
+    sample_tool.meta = {"original": True, "version": "1.0"}
+    transformed = Tool.from_tool(sample_tool)
+    assert transformed.meta == {"original": True, "version": "1.0"}
+
+
+def test_transform_overrides_meta(sample_tool):
+    """Test that transformed tools can override meta."""
+    sample_tool.meta = {"original": True, "version": "1.0"}
+    transformed = Tool.from_tool(sample_tool, meta={"custom": True, "priority": "high"})
+    assert transformed.meta == {"custom": True, "priority": "high"}
+
+
+def test_transform_sets_meta_to_none(sample_tool):
+    """Test that transformed tools can explicitly set meta to None."""
+    sample_tool.meta = {"original": True, "version": "1.0"}
+    transformed = Tool.from_tool(sample_tool, meta=None)
+    assert transformed.meta is None
+
+
+def test_transform_inherits_none_meta(sample_tool_no_title):
+    """Test that transformed tools inherit None meta."""
+    sample_tool_no_title.meta = None
+    transformed = Tool.from_tool(sample_tool_no_title)
+    assert transformed.meta is None
+
+
+def test_transform_adds_meta_to_none(sample_tool_no_title):
+    """Test that transformed tools can add meta when parent has None."""
+    sample_tool_no_title.meta = None
+    transformed = Tool.from_tool(sample_tool_no_title, meta={"added": True})
+    assert transformed.meta == {"added": True}
+
+
+def test_tool_transform_config_inherits_meta(sample_tool):
+    """Test that ToolTransformConfig inherits meta when unset."""
+    sample_tool.meta = {"original": True, "version": "1.0"}
+    config = ToolTransformConfig(name="config_tool")
+    transformed = config.apply(sample_tool)
+    assert transformed.meta == {"original": True, "version": "1.0"}
+
+
+def test_tool_transform_config_overrides_meta(sample_tool):
+    """Test that ToolTransformConfig can override meta."""
+    sample_tool.meta = {"original": True, "version": "1.0"}
+    config = ToolTransformConfig(
+        name="config_tool", meta={"config": True, "priority": "high"}
+    )
+    transformed = config.apply(sample_tool)
+    assert transformed.meta == {"config": True, "priority": "high"}
+
+
+def test_tool_transform_config_removes_meta(sample_tool):
+    """Test that ToolTransformConfig can remove meta with None."""
+    sample_tool.meta = {"original": True, "version": "1.0"}
+    config = ToolTransformConfig(name="config_tool", meta=None)
+    transformed = config.apply(sample_tool)
+    assert transformed.meta is None
+
+
+class TestInputSchema:
+    """Test schema definition handling and reference finding."""
+
+    def test_arg_transform_examples_in_schema(self, add_tool: Tool):
+        # Simple example
+        new_tool = Tool.from_tool(
+            add_tool,
+            transform_args={
+                "old_x": ArgTransform(examples=[1, 2, 3]),
+            },
+        )
+        prop = get_property(new_tool, "old_x")
+        assert prop["examples"] == [1, 2, 3]
+
+        # Nested example (e.g., for array type)
+        new_tool2 = Tool.from_tool(
+            add_tool,
+            transform_args={
+                "old_x": ArgTransform(examples=[["a", "b"], ["c", "d"]]),
+            },
+        )
+        prop2 = get_property(new_tool2, "old_x")
+        assert prop2["examples"] == [["a", "b"], ["c", "d"]]
+
+        # If not set, should not be present
+        new_tool3 = Tool.from_tool(
+            add_tool,
+            transform_args={
+                "old_x": ArgTransform(),
+            },
+        )
+        prop3 = get_property(new_tool3, "old_x")
+        assert "examples" not in prop3
+
+    def test_merge_schema_with_defs_precedence(self):
+        """Test _merge_schema_with_precedence merges $defs correctly."""
+        base_schema = {
+            "type": "object",
+            "properties": {"field1": {"$ref": "#/$defs/BaseType"}},
+            "$defs": {
+                "BaseType": {"type": "string", "description": "base"},
+                "SharedType": {"type": "integer", "minimum": 0},
+            },
+        }
+
+        override_schema = {
+            "type": "object",
+            "properties": {"field2": {"$ref": "#/$defs/OverrideType"}},
+            "$defs": {
+                "OverrideType": {"type": "boolean"},
+                "SharedType": {"type": "integer", "minimum": 10},  # Override
+            },
+        }
+
+        transformed_tool_schema = TransformedTool._merge_schema_with_precedence(
+            base_schema, override_schema
+        )
+
+        # SharedType should no longer be present on the schema
+        assert "SharedType" not in transformed_tool_schema["$defs"]
+
+        assert transformed_tool_schema == snapshot(
+            {
+                "type": "object",
+                "properties": {
+                    "field1": {"$ref": "#/$defs/BaseType"},
+                    "field2": {"$ref": "#/$defs/OverrideType"},
+                },
+                "required": [],
+                "$defs": {
+                    "BaseType": {"type": "string", "description": "base"},
+                    "OverrideType": {"type": "boolean"},
+                },
+            }
+        )
+
+    def test_transform_tool_with_complex_defs_pruning(self):
+        """Test that tool transformation properly prunes unused $defs."""
+
+        class UsedType(BaseModel):
+            value: str
+
+        class UnusedType(BaseModel):
+            other: int
+
+        @Tool.from_function
+        def complex_tool(
+            used_param: UsedType, unused_param: UnusedType | None = None
+        ) -> str:
+            return used_param.value
+
+        # Transform to hide unused_param
+        transformed_tool: TransformedTool = Tool.from_tool(
+            complex_tool, transform_args={"unused_param": ArgTransform(hide=True)}
+        )
+
+        assert "UnusedType" not in transformed_tool.parameters["$defs"]
+
+        assert transformed_tool.parameters == snapshot(
+            {
+                "type": "object",
+                "properties": {
+                    "used_param": {"$ref": "#/$defs/UsedType", "title": "Used Param"}
+                },
+                "required": ["used_param"],
+                "$defs": {
+                    "UsedType": {
+                        "properties": {"value": {"title": "Value", "type": "string"}},
+                        "required": ["value"],
+                        "title": "UsedType",
+                        "type": "object",
+                    }
+                },
+            }
+        )
+
+    def test_transform_with_custom_function_preserves_needed_defs(self):
+        """Test that custom transform functions preserve necessary $defs."""
+
+        class InputType(BaseModel):
+            data: str
+
+        class OutputType(BaseModel):
+            result: str
+
+        @Tool.from_function
+        def base_tool(input_data: InputType) -> OutputType:
+            return OutputType(result=input_data.data.upper())
+
+        async def transform_function(renamed_input: InputType):
+            return await forward(renamed_input=renamed_input)
+
+        # Transform with custom function and argument rename
+        transformed = Tool.from_tool(
+            base_tool,
+            transform_fn=transform_function,
+            transform_args={"input_data": ArgTransform(name="renamed_input")},
+        )
+
+        assert transformed.parameters == snapshot(
+            {
+                "type": "object",
+                "properties": {
+                    "renamed_input": {
+                        "$ref": "#/$defs/InputType",
+                        "title": "Input Data",
+                    }
+                },
+                "required": ["renamed_input"],
+                "$defs": {
+                    "InputType": {
+                        "properties": {"data": {"title": "Data", "type": "string"}},
+                        "required": ["data"],
+                        "title": "InputType",
+                        "type": "object",
+                    }
+                },
+            }
+        )
+
+    def test_chained_transforms_preserve_correct_defs(self):
+        """Test that chained transformations preserve correct $defs."""
+
+        class TypeA(BaseModel):
+            a: str
+
+        class TypeB(BaseModel):
+            b: int
+
+        class TypeC(BaseModel):
+            c: bool
+
+        @Tool.from_function
+        def base_tool(param_a: TypeA, param_b: TypeB, param_c: TypeC) -> str:
+            return f"{param_a.a}-{param_b.b}-{param_c.c}"
+
+        # First transform: hide param_c
+        transform1 = Tool.from_tool(
+            base_tool,
+            transform_args={"param_c": ArgTransform(hide=True, default=TypeC(c=True))},
+        )
+
+        assert transform1.parameters == snapshot(
+            {
+                "type": "object",
+                "properties": {
+                    "param_a": {"$ref": "#/$defs/TypeA", "title": "Param A"},
+                    "param_b": {"$ref": "#/$defs/TypeB", "title": "Param B"},
+                },
+                "required": IsList("param_b", "param_a", check_order=False),
+                "$defs": {
+                    "TypeA": {
+                        "properties": {"a": {"title": "A", "type": "string"}},
+                        "required": ["a"],
+                        "title": "TypeA",
+                        "type": "object",
+                    },
+                    "TypeB": {
+                        "properties": {"b": {"title": "B", "type": "integer"}},
+                        "required": ["b"],
+                        "title": "TypeB",
+                        "type": "object",
+                    },
+                },
+            }
+        )
+
+        assert "TypeA" in transform1.parameters["$defs"]
+
+        # Second transform: hide param_b
+        transform2 = Tool.from_tool(
+            transform1,
+            transform_args={"param_b": ArgTransform(hide=True, default=TypeB(b=42))},
+        )
+
+        assert "TypeB" not in transform2.parameters["$defs"]
+
+        assert transform2.parameters == snapshot(
+            {
+                "type": "object",
+                "properties": {
+                    "param_a": {"$ref": "#/$defs/TypeA", "title": "Param A"}
+                },
+                "required": ["param_a"],
+                "$defs": {
+                    "TypeA": {
+                        "properties": {"a": {"title": "A", "type": "string"}},
+                        "required": ["a"],
+                        "title": "TypeA",
+                        "type": "object",
+                    }
+                },
+            }
+        )
